@@ -7,8 +7,10 @@ Authors
  * Ju-Chieh Chou 2020
  * Samuele Cornell 2020
  * Abdel HEBA 2020
+ * Gaelle Laperriere 2021
+ * Sahar Ghannay 2021
+ * Sylvain de Langen 2022
 """
-
 import os
 import torch
 import logging
@@ -164,26 +166,41 @@ def read_audio(waveforms_obj):
     Expected use case is in conjunction with Datasets
     specified by JSON.
 
-    The custom notation:
+    The parameter may just be a path to a file:
+    `read_audio("/path/to/wav1.wav")`
 
-    The annotation can be just a path to a file:
-    "/path/to/wav1.wav"
+    Alternatively, you can specify more options in a dict, e.g.:
+    ```
+    # load a file from sample 8000 through 15999
+    read_audio({
+        "file": "/path/to/wav2.wav",
+        "start": 8000,
+        "stop": 16000
+    })
+    ```
 
-    Or can specify more options in a dict:
-    {"file": "/path/to/wav2.wav",
-    "start": 8000,
-    "stop": 16000
-    }
+    Which codecs are supported depends on your torchaudio backend.
+    Refer to `torchaudio.load` documentation for further details.
 
     Arguments
     ----------
     waveforms_obj : str, dict
-        Audio reading annotation, see above for format.
+        Path to audio or dict with the desired configuration.
+
+        Keys for the dict variant:
+        - `"file"` (str): Path to the audio file.
+        - `"start"` (int, optional): The first sample to load.
+        If unspecified, load from the very first frame.
+        - `"stop"` (int, optional): The last sample to load (exclusive).
+        If unspecified or equal to start, load from `start` to the end.
+        Will not fail if `stop` is past the sample count of the file and will
+        return less frames.
 
     Returns
     -------
     torch.Tensor
-        Audio tensor with shape: (samples, ).
+        1-channel: audio tensor with shape: `(samples, )`.
+        >=2-channels: audio tensor with shape: `(samples, channels)`.
 
     Example
     -------
@@ -198,15 +215,37 @@ def read_audio(waveforms_obj):
     """
     if isinstance(waveforms_obj, str):
         audio, _ = torchaudio.load(waveforms_obj)
-        return audio.transpose(0, 1).squeeze(1)
+    else:
+        path = waveforms_obj["file"]
+        start = waveforms_obj.get("start", 0)
+        # To match past SB behavior, `start == stop` or omitted `stop` means to
+        # load all frames from `start` to the file end.
+        stop = waveforms_obj.get("stop", start)
+
+        if start < 0:
+            raise ValueError(
+                f"Invalid sample range (start < 0): {start}..{stop}!"
+            )
+
+        if stop < start:
+            # Could occur if the user tried one of two things:
+            # - specify a negative value as an attempt to index from the end;
+            # - specify -1 as an attempt to load up to the last sample.
+            raise ValueError(
+                f"Invalid sample range (stop < start): {start}..{stop}!\n"
+                'Hint: Omit "stop" if you want to read to the end of file.'
+            )
+
+        # Requested to load until a specific frame?
+        if start != stop:
+            num_frames = stop - start
+            audio, fs = torchaudio.load(
+                path, num_frames=num_frames, frame_offset=start
+            )
+        else:
+            # Load to the end.
+            audio, fs = torchaudio.load(path, frame_offset=start)
 
-    path = waveforms_obj["file"]
-    start = waveforms_obj.get("start", 0)
-    # Default stop to start -> if not specified, num_frames becomes 0,
-    # which is the torchaudio default
-    stop = waveforms_obj.get("stop", start)
-    num_frames = stop - start
-    audio, fs = torchaudio.load(path, num_frames=num_frames, frame_offset=start)
     audio = audio.transpose(0, 1)
     return audio.squeeze(1)
 
@@ -1018,9 +1057,9 @@ def split_word(sequences, space="_"):
 
     Arguments
     ---------
-    sequences : list
+    sequences: list
         Each item contains a list, and this list contains a words sequence.
-    space : string
+    space: string
         The token represents space. Default: _
 
     Returns
@@ -1039,3 +1078,79 @@ def split_word(sequences, space="_"):
         chars = list(space.join(seq))
         results.append(chars)
     return results
+
+
+def extract_concepts_values(sequences, keep_values, tag_in, tag_out, space):
+    """keep the semantic concepts and values for evaluation.
+
+    Arguments
+    ---------
+    sequences: list
+        Each item contains a list, and this list contains a character sequence.
+    keep_values: bool
+        If True, keep the values. If not don't.
+    tag_in: char
+        Indicates the start of the concept.
+    tag_out: char
+        Indicates the end of the concept.
+    space: string
+        The token represents space. Default: _
+
+    Returns
+    -------
+    The list contains concept and value sequences for each sentence.
+
+    Example
+    -------
+    >>> sequences = [['<reponse>','_','n','o','_','>','_','<localisation-ville>','_','L','e','_','M','a','n','s','_','>'], ['<reponse>','_','s','i','_','>'],['v','a','_','b','e','n','e']]
+    >>> results = extract_concepts_values(sequences, True, '<', '>', '_')
+    >>> results
+    [['<reponse> no', '<localisation-ville> Le Mans'], ['<reponse> si'], ['']]
+    """
+    results = []
+    for sequence in sequences:
+        # ['<reponse>_no_>_<localisation-ville>_Le_Mans_>']
+        sequence = "".join(sequence)
+        # ['<reponse>','no','>','<localisation-ville>','Le','Mans,'>']
+        sequence = sequence.split(space)
+        processed_sequence = []
+        value = (
+            []
+        )  # If previous sequence value never used because never had a tag_out
+        kept = ""  # If previous sequence kept never used because never had a tag_out
+        concept_open = False
+        for word in sequence:
+            if re.match(tag_in, word):
+                # If not close tag but new tag open
+                if concept_open and keep_values:
+                    if len(value) != 0:
+                        kept += " " + " ".join(value)
+                    concept_open = False
+                    processed_sequence.append(kept)
+                kept = word  # 1st loop: '<reponse>'
+                value = []  # Concept's value
+                concept_open = True  # Trying to catch the concept's value
+                # If we want the CER
+                if not keep_values:
+                    processed_sequence.append(kept)  # Add the kept concept
+            # If we have a tag_out, had a concept, and want the values for CVER
+            elif re.match(tag_out, word) and concept_open and keep_values:
+                # If we have a value
+                if len(value) != 0:
+                    kept += " " + " ".join(
+                        value
+                    )  # 1st loop: '<response>' + ' ' + 'no'
+                concept_open = False  # Wait for a new tag_in to pursue
+                processed_sequence.append(kept)  # Add the kept concept + value
+            elif concept_open:
+                value.append(word)  # 1st loop: 'no'
+        # If not close tag but end sequence
+        if concept_open and keep_values:
+            if len(value) != 0:
+                kept += " " + " ".join(value)
+            concept_open = False
+            processed_sequence.append(kept)
+        if len(processed_sequence) == 0:
+            processed_sequence.append("")
+        results.append(processed_sequence)
+    return results
