@@ -43,13 +43,13 @@ class SpecAugment(torch.nn.Module):
         Time warp window.
     time_warp_mode : str
         Interpolation mode for time warping (default "bicubic").
-    freq_mask : bool1
+    freq_mask : bool
         Whether applying freq mask.
     freq_mask_width : int or tuple
         Freq mask width range.
     n_freq_mask : int
         Number of freq mask.
-    time_mask : int
+    time_mask : bool
         Whether applying time mask.
     time_mask_width : int or tuple
         Time mask width range.
@@ -195,7 +195,8 @@ class SpecAugment(torch.nn.Module):
         if self.replace_with_zero:
             val = 0.0
         else:
-            val = x.mean()
+            with torch.no_grad():
+                val = x.mean()
 
         x = x.masked_fill_(mask, val)
         return x.view(*original_size)
@@ -401,6 +402,7 @@ class EnvCorrupt(torch.nn.Module):
             self.add_reverb = AddReverb(
                 reverb_prob=reverb_prob,
                 csv_file=reverb_csv,
+                rir_scale_factor=rir_scale_factor,
                 reverb_sample_rate=reverb_sample_rate,
                 clean_sample_rate=clean_sample_rate,
             )
@@ -502,6 +504,8 @@ def _prepare_csv(folder, filelist, csv_file, max_length=None):
         than this will be cut into pieces.
     """
     try:
+        # make sure all processing reached here before main preocess create csv_file
+        sb.utils.distributed.ddp_barrier()
         if sb.utils.distributed.if_main_process():
             with open(csv_file, "w") as w:
                 w.write("ID,duration,wav,wav_format,wav_opts\n\n")
