@@ -28,10 +28,19 @@ def xavier_init_layer(
 
 
 def shifted_sigmoid(x):
+    "Computes the shifted sigmoid."
     return 1.2 / (1 + torch.exp(-(1 / 1.6) * x))
 
 
 class Learnable_sigmoid(nn.Module):
+    """Implementation of a leanable sigmoid.
+
+    Arguments
+    ---------
+    in_features : int
+        Input dimensionality
+    """
+
     def __init__(self, in_features=257):
         super().__init__()
         self.slope = nn.Parameter(torch.ones(in_features))
@@ -41,6 +50,7 @@ class Learnable_sigmoid(nn.Module):
         # self.scale.requiresGrad = True # set requiresGrad to true!
 
     def forward(self, x):
+        """ Processes the input tensor x and returns an output tensor."""
         return 1.2 * torch.sigmoid(self.slope * x)
 
 
@@ -91,6 +101,7 @@ class EnhancementGenerator(nn.Module):
         self.sigmoid = nn.Sigmoid()
 
     def forward(self, x, lengths):
+        """ Processes the input tensor x and returns an output tensor."""
         out, _ = self.blstm(x, lengths=lengths)
 
         out = self.linear1(out)
@@ -145,6 +156,7 @@ class MetricDiscriminator(nn.Module):
         self.Linear3 = xavier_init_layer(in_size=10, out_size=1)
 
     def forward(self, x):
+        """ Processes the input tensor x and returns an output tensor."""
         out = self.BN(x)
 
         out = self.conv1(out)
