@@ -421,6 +421,13 @@ class DepthwiseSeparableConv(sb.nnet.containers.Sequential):
         batchsize, time, in_channels = input_shape
 
         # [M, K, H] -> [M, K, H]
+        if causal:
+            paddingval = dilation * (kernel_size - 1)
+            padding = "causal"
+            default_padding = "same"
+        else:
+            default_padding = 0
+
         self.append(
             sb.nnet.CNN.Conv1d,
             out_channels=in_channels,
@@ -431,10 +438,11 @@ class DepthwiseSeparableConv(sb.nnet.containers.Sequential):
             groups=in_channels,
             bias=False,
             layer_name="conv_0",
+            default_padding=default_padding,
         )
 
         if causal:
-            self.append(Chomp1d(padding), layer_name="chomp")
+            self.append(Chomp1d(paddingval), layer_name="chomp")
 
         self.append(nn.PReLU(), layer_name="act")
         self.append(choose_norm(norm_type, in_channels), layer_name="act")
