@@ -59,6 +59,9 @@ class ConvolutionFrontEnd(Sequential):
         activation=torch.nn.LeakyReLU,
         norm=LayerNorm,
         dropout=0.1,
+        conv_bias=True,
+        padding="same",
+        conv_init=None,
     ):
         super().__init__(input_shape=input_shape)
         for i in range(num_blocks):
@@ -75,6 +78,9 @@ class ConvolutionFrontEnd(Sequential):
                 norm=norm,
                 dropout=dropout,
                 layer_name=f"convblock_{i}",
+                conv_bias=conv_bias,
+                padding=padding,
+                conv_init=conv_init,
             )
 
 
@@ -120,9 +126,11 @@ class ConvBlock(torch.nn.Module):
         activation=torch.nn.LeakyReLU,
         norm=None,
         dropout=0.1,
+        conv_bias=True,
+        padding="same",
+        conv_init=None,
     ):
         super().__init__()
-
         self.convs = Sequential(input_shape=input_shape)
 
         for i in range(num_layers):
@@ -133,6 +141,9 @@ class ConvBlock(torch.nn.Module):
                 stride=stride if i == num_layers - 1 else 1,
                 dilation=dilation,
                 layer_name=f"conv_{i}",
+                bias=conv_bias,
+                padding=padding,
+                conv_init=conv_init,
             )
             if norm is not None:
                 self.convs.append(norm, layer_name=f"norm_{i}")
@@ -161,5 +172,4 @@ class ConvBlock(torch.nn.Module):
         if self.reduce_conv:
             out = out + self.reduce_conv(x)
             out = self.drop(out)
-
         return out
