@@ -16,8 +16,11 @@ import copy
 from speechbrain.nnet.linear import Linear
 from speechbrain.lobes.models.transformer.Transformer import TransformerEncoder
 from speechbrain.lobes.models.transformer.Transformer import PositionalEncoding
+from speechbrain.lobes.models.transformer.Conformer import ConformerEncoder
 import speechbrain.nnet.RNN as SBRNN
 
+from speechbrain.nnet.activations import Swish
+
 
 EPS = 1e-8
 
@@ -113,9 +116,9 @@ class CumulativeLayerNorm(nn.LayerNorm):
     >>> x_norm = CLN(x)
     """
 
-    def __init__(self, dim, elementwise_affine=True):
+    def __init__(self, dim, elementwise_affine=True, eps=1e-8):
         super(CumulativeLayerNorm, self).__init__(
-            dim, elementwise_affine=elementwise_affine, eps=1e-8
+            dim, elementwise_affine=elementwise_affine, eps=eps
         )
 
     def forward(self, x):
@@ -143,16 +146,16 @@ class CumulativeLayerNorm(nn.LayerNorm):
         return x
 
 
-def select_norm(norm, dim, shape):
+def select_norm(norm, dim, shape, eps=1e-8):
     """Just a wrapper to select the normalization type.
     """
 
     if norm == "gln":
-        return GlobalLayerNorm(dim, shape, elementwise_affine=True)
+        return GlobalLayerNorm(dim, shape, elementwise_affine=True, eps=eps)
     if norm == "cln":
-        return CumulativeLayerNorm(dim, elementwise_affine=True)
+        return CumulativeLayerNorm(dim, elementwise_affine=True, eps=eps)
     if norm == "ln":
-        return nn.GroupNorm(1, dim, eps=1e-8)
+        return nn.GroupNorm(1, dim, eps=eps)
     else:
         return nn.BatchNorm1d(dim)
 
@@ -1064,10 +1067,18 @@ class Dual_Path_Model(nn.Module):
         P = K // 2
         gap = K - (P + L % K) % K
         if gap > 0:
-            pad = torch.Tensor(torch.zeros(B, N, gap)).type(input.type())
+            pad = (
+                torch.Tensor(torch.zeros(B, N, gap))
+                .type(input.dtype)
+                .to(input.device)
+            )
             input = torch.cat([input, pad], dim=2)
 
-        _pad = torch.Tensor(torch.zeros(B, N, P)).type(input.type())
+        _pad = (
+            torch.Tensor(torch.zeros(B, N, P))
+            .type(input.dtype)
+            .to(input.device)
+        )
         input = torch.cat([_pad, input, _pad], dim=2)
 
         return input, gap
@@ -1280,7 +1291,7 @@ class SepformerWrapper(nn.Module):
                 self.reset_layer_recursively(child_layer)
 
     def forward(self, mix):
-
+        """ Processes the input tensor x and returns an output tensor."""
         mix_w = self.encoder(mix)
         est_mask = self.masknet(mix_w)
         mix_w = torch.stack([mix_w] * self.num_spks)
@@ -1304,3 +1315,127 @@ class SepformerWrapper(nn.Module):
             est_source = est_source[:, :T_origin, :]
 
         return est_source
+
+
+class SBConformerEncoderBlock(nn.Module):
+    """A wrapper for the SpeechBrain implementation of the ConformerEncoder.
+
+    Arguments
+    ---------
+    num_layers : int
+        Number of layers.
+    d_model : int
+        Dimensionality of the representation.
+    nhead : int
+        Number of attention heads.
+    d_ffn : int
+        Dimensionality of positional feed forward.
+    input_shape : tuple
+        Shape of input.
+    kdim : int
+        Dimension of the key (Optional).
+    vdim : int
+        Dimension of the value (Optional).
+    dropout : float
+        Dropout rate.
+    activation : str
+        Activation function.
+    kernel_size: int
+        Kernel size in the conformer encoder
+    bias: bool
+        Use bias or not in the convolution part of conformer encoder
+    use_positional_encoding : bool
+        If true we use a positional encoding.
+
+
+    Example
+    ---------
+    >>> x = torch.randn(10, 100, 64)
+    >>> block = SBConformerEncoderBlock(1, 64, 8)
+    >>> from speechbrain.lobes.models.transformer.Transformer import PositionalEncoding
+    >>> pos_enc = PositionalEncoding(64)
+    >>> pos_embs = pos_enc(torch.ones(1, 199, 64))
+    >>> x = block(x)
+    >>> x.shape
+    torch.Size([10, 100, 64])
+    """
+
+    def __init__(
+        self,
+        num_layers,
+        d_model,
+        nhead,
+        d_ffn=2048,
+        input_shape=None,
+        kdim=None,
+        vdim=None,
+        dropout=0.1,
+        activation="swish",
+        kernel_size=31,
+        bias=True,
+        use_positional_encoding=True,
+        attention_type="RelPosMHAXL",
+    ):
+        super(SBConformerEncoderBlock, self).__init__()
+        self.use_positional_encoding = use_positional_encoding
+        self.attention_type = attention_type
+
+        if activation == "relu":
+            activation = nn.ReLU
+        elif activation == "gelu":
+            activation = nn.GELU
+        elif activation == "swish":
+            activation = Swish
+        else:
+            raise ValueError("unknown activation")
+
+        self.mdl = ConformerEncoder(
+            num_layers=num_layers,
+            nhead=nhead,
+            d_ffn=d_ffn,
+            d_model=d_model,
+            kdim=kdim,
+            vdim=vdim,
+            dropout=dropout,
+            activation=activation,
+            kernel_size=kernel_size,
+            bias=bias,
+            attention_type=attention_type,
+        )
+
+        if self.attention_type == "RelPosMHAXL":
+            # for RelPosMHAXL, we need the positional encoding (not optional)
+            self.pos_enc = PositionalEncoding(input_size=d_model)
+        elif self.attention_type == "regularMHA":
+            if self.use_positional_encoding:
+                self.pos_enc = PositionalEncoding(input_size=d_model)
+        else:
+            raise ValueError("Unsupported attention type")
+
+    def forward(self, x):
+        """Returns the transformed output.
+
+        Arguments
+        ---------
+        x : torch.Tensor
+            Tensor shape [B, L, N],
+            where, B = Batchsize,
+                   L = time points
+                   N = number of filters
+
+        """
+        if self.attention_type == "RelPosMHAXL":
+            pos_enc = self.pos_enc(
+                torch.ones(
+                    x.shape[0], x.shape[1] * 2 - 1, x.shape[2], device=x.device
+                )
+            )
+            return self.mdl(x, pos_embs=pos_enc)[0]
+        elif self.attention_type == "regularMHA":
+            if self.use_positional_encoding:
+                pos_embs = self.pos_enc(x)
+                return self.mdl(x + pos_embs)[0]
+            else:
+                return self.mdl(x)[0]
+        else:
+            raise ValueError("Unsupported attention type")
