@@ -20,6 +20,7 @@ import torch.nn.functional as F
 from torch import nn
 from huggingface_hub import model_info
 from speechbrain.pretrained.fetching import fetch
+from speechbrain.dataio.dataio import length_to_mask
 
 # We check if transformers is installed.
 try:
@@ -83,6 +84,12 @@ class HuggingFaceWav2Vec2(nn.Module):
         If True, the model will apply spec augment on the output of feature extractor
         (inside huggingface Wav2VecModel() class).
         If False, the model will not apply spec augment. We set this to false to prevent from doing it twice.
+    output_all_hiddens : bool (default: False)
+        If True, the forward function outputs the hidden states from all transformer layers.
+        For example wav2vec2-base has 12 transformer layers and the output is of shape (13, B, T, C),
+        where a projection of the CNN output is added to the beginning.
+        If False, the forward function outputs the hidden states only from the last transformer layer.
+
     Example
     -------
     >>> inputs = torch.rand([10, 600])
@@ -96,10 +103,11 @@ class HuggingFaceWav2Vec2(nn.Module):
         self,
         source,
         save_path,
-        output_norm=True,
-        freeze=True,
+        output_norm=False,
+        freeze=False,
         freeze_feature_extractor=False,
         apply_spec_augment=False,
+        output_all_hiddens=False,
     ):
         super().__init__()
 
@@ -125,7 +133,6 @@ class HuggingFaceWav2Vec2(nn.Module):
             source, config=config, model=model, save_path=save_path
         )
 
-        # set apply_spec_augment
         self.model.config.apply_spec_augment = apply_spec_augment
 
         # We check if inputs need to be normalized w.r.t pretrained wav2vec2
@@ -144,7 +151,13 @@ class HuggingFaceWav2Vec2(nn.Module):
         else:
             self.model.train()
             if self.freeze_feature_extractor:
-                self.model.feature_extractor._freeze_parameters()
+                logger.warning(
+                    "speechbrain.lobes.models.huggingface_wav2vec - wav2vec 2.0 feature extractor is frozen."
+                )
+                self.model.feature_extractor.eval()
+                for param in self.model.feature_extractor.parameters():
+                    param.requires_grad = False
+        self.output_all_hiddens = output_all_hiddens
 
     def _from_pretrained(self, source, config, model, save_path):
         """This function manages the source checking and loading of the params.
@@ -153,7 +166,7 @@ class HuggingFaceWav2Vec2(nn.Module):
         # 3. Download (if appropriate) and load with respect to 1. and 2.
         """
 
-        is_sb, ckpt_file = self._check_model_source(source)
+        is_sb, ckpt_file, is_local = self._check_model_source(source, save_path)
         if is_sb:
             config = config.from_pretrained(source, cache_dir=save_path)
             self.model = model(config)
@@ -165,7 +178,9 @@ class HuggingFaceWav2Vec2(nn.Module):
             # We transfer the parameters from the checkpoint.
             self._load_sb_pretrained_w2v2_parameters(ckpt_full_path)
         else:
-            self.model = model.from_pretrained(source, cache_dir=save_path)
+            self.model = model.from_pretrained(
+                source, cache_dir=save_path, local_files_only=is_local
+            )
 
     def _load_sb_pretrained_w2v2_parameters(self, path):
         """Loads the parameter of a w2v2 model pretrained with SpeechBrain and the
@@ -203,31 +218,62 @@ class HuggingFaceWav2Vec2(nn.Module):
                 + "is useless for wav2vec 2.0 finetuning."
             )
 
-    def _check_model_source(self, path):
+    def _check_model_source(self, path, save_path):
         """Checks if the pretrained model has been trained with SpeechBrain and
         is hosted locally or on a HuggingFace hub.
+        Called as static function in HuggingFaceTransformer._from_pretrained.
+        Arguments
+        ---------
+        path : str
+            Used as "source"; local path or HuggingFace hub name: e.g "facebook/wav2vec2-large-lv60"
+        save_path : str
+            norm_output (dir) of the downloaded model.
+        Returns
+        -------
+        is_sb : bool
+            Whether/not the model is deserializable w/ SpeechBrain or not (then, model conversion is needed).
+        checkpoint_filename : str
+            as of HuggingFace documentation: file name relative to the repo root (guaranteed to be here).
         """
         checkpoint_filename = ""
         source = pathlib.Path(path)
         is_local = True
-        is_sb = True
 
         # If path is a huggingface hub.
         if not source.exists():
             is_local = False
 
+        # Check if source is downloaded already
+        sink = pathlib.Path(
+            save_path + "/models--" + path.replace("/", "--") + "/snapshots"
+        )
+        if sink.exists():
+            sink = (
+                sink / os.listdir(str(sink))[0]
+            )  # there's a hash-id subfolder
+            if any(
+                File.endswith(".bin") or File.endswith(".ckpt")
+                for File in os.listdir(str(sink))
+            ):
+                is_local = True
+                local_path = str(sink)
+            else:
+                local_path = path
+        else:
+            local_path = path
+
         if is_local:
             # Test for HuggingFace model
-            if any(File.endswith(".bin") for File in os.listdir(path)):
+            if any(File.endswith(".bin") for File in os.listdir(local_path)):
                 is_sb = False
-                return is_sb, checkpoint_filename
+                return is_sb, checkpoint_filename, is_local
 
             # Test for SpeechBrain model and get the filename.
-            for File in os.listdir(path):
+            for File in os.listdir(local_path):
                 if File.endswith(".ckpt"):
                     checkpoint_filename = os.path.join(path, File)
                     is_sb = True
-                    return is_sb, checkpoint_filename
+                    return is_sb, checkpoint_filename, is_local
         else:
             files = model_info(
                 path
@@ -238,54 +284,89 @@ class HuggingFaceWav2Vec2(nn.Module):
                 if File.rfilename.endswith(".ckpt"):
                     checkpoint_filename = File.rfilename
                     is_sb = True
-                    return is_sb, checkpoint_filename
+                    return is_sb, checkpoint_filename, is_local
 
             for File in files:
                 if File.rfilename.endswith(".bin"):
                     checkpoint_filename = File.rfilename
                     is_sb = False
-                    return is_sb, checkpoint_filename
+                    return is_sb, checkpoint_filename, is_local
 
         err_msg = f"{path} does not contain a .bin or .ckpt checkpoint !"
         raise FileNotFoundError(err_msg)
 
-    def forward(self, wav):
+    def forward(self, wav, wav_lens=None):
         """Takes an input waveform and return its corresponding wav2vec encoding.
 
         Arguments
         ---------
         wav : torch.Tensor (signal)
             A batch of audio signals to transform to features.
+        wav_len : tensor
+            The relative length of the wav given in SpeechBrain format.
         """
 
-        # If we freeze, we simply remove all grads and features from the graph.
+        # If we freeze, we simply remove all grads from the graph.
         if self.freeze:
             with torch.no_grad():
-                return self.extract_features(wav).detach()
+                return self.extract_features(wav, wav_lens)
 
-        return self.extract_features(wav)
+        return self.extract_features(wav, wav_lens)
 
-    def extract_features(self, wav):
+    def extract_features(self, wav, wav_lens=None):
         """Takes an input waveform and return its corresponding wav2vec encoding.
 
         Arguments
         ---------
         wav : torch.Tensor (signal)
             A batch of audio signals to transform to features.
+        wav_len : tensor
+            The relative length of the wav given in SpeechBrain format.
         """
 
+        padding_mask = self.make_masks(wav, wav_len=wav_lens)
+
         if self.normalize_wav:
-            wav = F.layer_norm(wav, wav.shape)
+            wav = F.layer_norm(wav, wav.shape[1:])
 
         # Extract wav2vec output
-        out = self.model(wav)[0]
+        out = self.model(
+            wav,
+            attention_mask=padding_mask,
+            output_hidden_states=self.output_all_hiddens,
+        )
+
+        if self.output_all_hiddens:
+            out = torch.stack(list(out.hidden_states), dim=0)
+            norm_shape = out.shape[-3:]
+        else:
+            out = out.last_hidden_state
+            norm_shape = out.shape
 
         # We normalize the output if required
         if self.output_norm:
-            out = F.layer_norm(out, out.shape)
+            out = F.layer_norm(out, norm_shape[1:])
 
         return out
 
+    def make_masks(self, src, wav_len=None, pad_idx=0):
+        """This method generates the padding masks.
+        Arguments
+        ---------
+        src : tensor
+            The sequence to the encoder (required).
+        wav_len : tensor
+            The relative length of the wav given in SpeechBrain format.
+        pad_idx : int
+            The index for <pad> token (default=0).
+        """
+        src_key_padding_mask = None
+        if wav_len is not None:
+            abs_len = torch.round(wav_len * src.shape[1])
+            src_key_padding_mask = length_to_mask(abs_len).bool()
+
+        return src_key_padding_mask
+
 
 class HuggingFaceWav2Vec2Pretrain(nn.Module):
     """This lobe enables the integration of HuggingFace
@@ -348,13 +429,15 @@ class HuggingFaceWav2Vec2Pretrain(nn.Module):
 
         # We check if inputs need to be normalized w.r.t pretrained wav2vec2
 
-    def forward(self, wav):
+    def forward(self, wav, wav_lens):
         """Takes an input waveform and return its corresponding wav2vec encoding.
 
         Arguments
         ---------
         wav : torch.Tensor (signal)
             A batch of audio signals to transform to features.
+        wav_len : tensor
+            The relative length of the wav given in SpeechBrain format.
         """
         batch_size, raw_sequence_length = wav.shape
 
@@ -363,7 +446,7 @@ class HuggingFaceWav2Vec2Pretrain(nn.Module):
 
         sequence_length = self.model._get_feat_extract_output_lengths(
             raw_sequence_length
-        )
+        ).item()
 
         # 1. Compute the indices that will be masked
         mask_time_indices = _compute_mask_indices(
@@ -374,6 +457,7 @@ class HuggingFaceWav2Vec2Pretrain(nn.Module):
         torch_mask_time_indices = torch.tensor(
             mask_time_indices, device=wav.device, dtype=torch.long,
         )
+        padding_mask = self.make_padding_masks(wav, wav_len=wav_lens)
 
         # 2. Sample the negative samples from the entire sequence.
         # Fairseq does it only on the masked indices, but this only work if you
@@ -397,6 +481,25 @@ class HuggingFaceWav2Vec2Pretrain(nn.Module):
                 wav,
                 mask_time_indices=torch_mask_time_indices,
                 sampled_negative_indices=negative_sample_indices,
+                attention_mask=padding_mask,
             ),
             torch_mask_time_indices,
         )
+
+    def make_padding_masks(self, src, wav_len=None, pad_idx=0):
+        """This method generates the padding masks.
+        Arguments
+        ---------
+        src : tensor
+            The sequence to the encoder (required).
+        wav_len : tensor
+            The relative length of the wav given in SpeechBrain format.
+        pad_idx : int
+            The index for <pad> token (default=0).
+        """
+        src_key_padding_mask = None
+        if wav_len is not None:
+            abs_len = torch.round(wav_len * src.shape[1])
+            src_key_padding_mask = length_to_mask(abs_len).bool()
+
+        return src_key_padding_mask
