@@ -9,6 +9,7 @@ import torch
 import torch.nn as nn
 import speechbrain as sb
 from typing import Optional
+import numpy as np
 
 
 from .Conformer import ConformerEncoder
@@ -339,6 +340,7 @@ class TransformerEncoderLayer(nn.Module):
         src_key_padding_mask : torch.Tensor, optional
             The mask for the src keys for each example in the batch.
         """
+
         if self.normalize_before:
             src1 = self.norm1(src)
         else:
@@ -368,7 +370,6 @@ class TransformerEncoderLayer(nn.Module):
         output = src + self.dropout2(output)
         if not self.normalize_before:
             output = self.norm2(output)
-
         return output, self_attn
 
 
@@ -418,6 +419,7 @@ class TransformerEncoder(nn.Module):
         activation=nn.ReLU,
         normalize_before=False,
         causal=False,
+        layerdrop_prob=0.0,
         attention_type="regularMHA",
     ):
         super().__init__()
@@ -440,6 +442,8 @@ class TransformerEncoder(nn.Module):
             ]
         )
         self.norm = sb.nnet.normalization.LayerNorm(d_model, eps=1e-6)
+        self.layerdrop_prob = layerdrop_prob
+        self.rng = np.random.default_rng()
 
     def forward(
         self,
@@ -459,17 +463,26 @@ class TransformerEncoder(nn.Module):
             The mask for the src keys per batch (optional).
         """
         output = src
+        if self.layerdrop_prob > 0.0:
+            keep_probs = self.rng.random(len(self.layers))
+        else:
+            keep_probs = None
         attention_lst = []
-        for enc_layer in self.layers:
-            output, attention = enc_layer(
-                output,
-                src_mask=src_mask,
-                src_key_padding_mask=src_key_padding_mask,
-                pos_embs=pos_embs,
-            )
-            attention_lst.append(attention)
-        output = self.norm(output)
+        for i, enc_layer in enumerate(self.layers):
+            if (
+                not self.training
+                or self.layerdrop_prob == 0.0
+                or keep_probs[i] > self.layerdrop_prob
+            ):
+                output, attention = enc_layer(
+                    output,
+                    src_mask=src_mask,
+                    src_key_padding_mask=src_key_padding_mask,
+                    pos_embs=pos_embs,
+                )
 
+                attention_lst.append(attention)
+        output = self.norm(output)
         return output, attention_lst
 
 
