@@ -5,6 +5,8 @@ Authors
  * Jianyuan Zhong 2020
  * Cem Subakan 2021
  * Davide Borra 2021
+ * Andreas Nautsch 2022
+ * Sarthak Yadav 2022
 """
 
 import math
@@ -13,7 +15,12 @@ import logging
 import numpy as np
 import torch.nn as nn
 import torch.nn.functional as F
+import torchaudio
 from typing import Tuple
+from speechbrain.processing.signal_processing import (
+    gabor_impulse_response,
+    gabor_impulse_response_legacy_complex,
+)
 
 logger = logging.getLogger(__name__)
 
@@ -83,6 +90,7 @@ class SincConv(nn.Module):
         min_band_hz=50,
     ):
         super().__init__()
+        self.in_channels = in_channels
         self.out_channels = out_channels
         self.kernel_size = kernel_size
         self.stride = stride
@@ -94,11 +102,16 @@ class SincConv(nn.Module):
         self.min_band_hz = min_band_hz
 
         # input shape inference
-        if input_shape is None and in_channels is None:
+        if input_shape is None and self.in_channels is None:
             raise ValueError("Must provide one of input_shape or in_channels")
 
-        if in_channels is None:
-            in_channels = self._check_input_shape(input_shape)
+        if self.in_channels is None:
+            self.in_channels = self._check_input_shape(input_shape)
+
+        if self.out_channels % self.in_channels != 0:
+            raise ValueError(
+                "Number of output channels must be divisible by in_channels"
+            )
 
         # Initialize Sinc filters
         self._init_sinc_conv()
@@ -145,6 +158,7 @@ class SincConv(nn.Module):
             stride=self.stride,
             padding=0,
             dilation=self.dilation,
+            groups=self.in_channels,
         )
 
         if unsqueeze:
@@ -155,13 +169,12 @@ class SincConv(nn.Module):
         return wx
 
     def _check_input_shape(self, shape):
-        """Checks the input shape and returns the number of input channels.
-        """
+        """Checks the input shape and returns the number of input channels."""
 
         if len(shape) == 2:
             in_channels = 1
         elif len(shape) == 3:
-            in_channels = 1
+            in_channels = shape[-1]
         else:
             raise ValueError(
                 "sincconv expects 2d or 3d inputs. Got " + str(len(shape))
@@ -176,8 +189,7 @@ class SincConv(nn.Module):
         return in_channels
 
     def _get_sinc_filters(self,):
-        """This functions creates the sinc-filters to used for sinc-conv.
-        """
+        """This functions creates the sinc-filters to used for sinc-conv."""
         # Computing the low frequencies of the filters
         low = self.min_low_hz + torch.abs(self.low_hz_)
 
@@ -257,13 +269,11 @@ class SincConv(nn.Module):
         )
 
     def _to_mel(self, hz):
-        """Converts frequency in Hz to the mel scale.
-        """
+        """Converts frequency in Hz to the mel scale."""
         return 2595 * np.log10(1 + hz / 700)
 
     def _to_hz(self, mel):
-        """Converts frequency in the mel scale to Hz.
-        """
+        """Converts frequency in the mel scale to Hz."""
         return 700 * (10 ** (mel / 2595) - 1)
 
     def _manage_padding(
@@ -285,7 +295,7 @@ class SincConv(nn.Module):
         """
 
         # Detecting input shape
-        L_in = x.shape[-1]
+        L_in = self.in_channels
 
         # Time padding
         padding = get_padding_elem(L_in, stride, kernel_size, dilation)
@@ -326,6 +336,11 @@ class Conv1d(nn.Module):
     skip_transpose : bool
         If False, uses batch x time x channel convention of speechbrain.
         If True, uses batch x channel x time convention.
+    weight_norm : bool
+        If True, use weight normalization,
+        to be removed with self.remove_weight_norm() at inference
+    default_padding: str or int
+        This sets the default padding mode that will be used by the pytorch Conv1d backend.
 
     Example
     -------
@@ -351,6 +366,9 @@ class Conv1d(nn.Module):
         bias=True,
         padding_mode="reflect",
         skip_transpose=False,
+        weight_norm=False,
+        conv_init=None,
+        default_padding=0,
     ):
         super().__init__()
         self.kernel_size = kernel_size
@@ -367,17 +385,25 @@ class Conv1d(nn.Module):
         if in_channels is None:
             in_channels = self._check_input_shape(input_shape)
 
+        self.in_channels = in_channels
+
         self.conv = nn.Conv1d(
             in_channels,
             out_channels,
             self.kernel_size,
             stride=self.stride,
             dilation=self.dilation,
-            padding=0,
+            padding=default_padding,
             groups=groups,
             bias=bias,
         )
 
+        if conv_init == "kaiming":
+            nn.init.kaiming_normal_(self.conv.weight)
+
+        if weight_norm:
+            self.conv = nn.utils.weight_norm(self.conv)
+
     def forward(self, x):
         """Returns the output of the convolution.
 
@@ -386,7 +412,6 @@ class Conv1d(nn.Module):
         x : torch.Tensor (batch, time, channel)
             input to convolve. 2d or 4d tensors are expected.
         """
-
         if not self.skip_transpose:
             x = x.transpose(1, -1)
 
@@ -440,7 +465,7 @@ class Conv1d(nn.Module):
         """
 
         # Detecting input shape
-        L_in = x.shape[-1]
+        L_in = self.in_channels
 
         # Time padding
         padding = get_padding_elem(L_in, stride, kernel_size, dilation)
@@ -451,8 +476,7 @@ class Conv1d(nn.Module):
         return x
 
     def _check_input_shape(self, shape):
-        """Checks the input shape and returns the number of input channels.
-        """
+        """Checks the input shape and returns the number of input channels."""
 
         if len(shape) == 2:
             self.unsqueeze = True
@@ -467,13 +491,18 @@ class Conv1d(nn.Module):
             )
 
         # Kernel size must be odd
-        if self.kernel_size % 2 == 0:
+        if not self.padding == "valid" and self.kernel_size % 2 == 0:
             raise ValueError(
                 "The field kernel size must be an odd number. Got %s."
                 % (self.kernel_size)
             )
+
         return in_channels
 
+    def remove_weight_norm(self):
+        """Removes weight normalization at inference if used during training."""
+        self.conv = nn.utils.remove_weight_norm(self.conv)
+
 
 class Conv2d(nn.Module):
     """This function implements 2d convolution.
@@ -496,8 +525,11 @@ class Conv2d(nn.Module):
         Dilation factor of the 2d convolutional filters over time and
         frequency axis.
     padding : str
-        (same, valid). If "valid", no padding is performed.
+        (same, valid, causal).
+        If "valid", no padding is performed.
         If "same" and stride is 1, output shape is same as input shape.
+        If "causal" then proper padding is inserted to simulate causal convolution on the first spatial dimension.
+        (spatial dim 1 is dim 3 for both skip_transpose=False and skip_transpose=True)
     padding_mode : str
         This flag specifies the type of padding. See torch.nn documentation
         for more information.
@@ -506,6 +538,12 @@ class Conv2d(nn.Module):
         documentation for more information.
     bias : bool
         If True, the additive bias b is adopted.
+    skip_transpose : bool
+        If False, uses batch x spatial.dim2 x spatial.dim1 x channel convention of speechbrain.
+        If True, uses batch x channel x spatial.dim1 x spatial.dim2 convention.
+    weight_norm : bool
+        If True, use weight normalization,
+        to be removed with self.remove_weight_norm() at inference
 
     Example
     -------
@@ -530,6 +568,9 @@ class Conv2d(nn.Module):
         groups=1,
         bias=True,
         padding_mode="reflect",
+        skip_transpose=False,
+        weight_norm=False,
+        conv_init=None,
     ):
         super().__init__()
 
@@ -547,6 +588,7 @@ class Conv2d(nn.Module):
         self.padding = padding
         self.padding_mode = padding_mode
         self.unsqueeze = False
+        self.skip_transpose = skip_transpose
 
         if input_shape is None and in_channels is None:
             raise ValueError("Must provide one of input_shape or in_channels")
@@ -554,9 +596,11 @@ class Conv2d(nn.Module):
         if in_channels is None:
             in_channels = self._check_input(input_shape)
 
+        self.in_channels = in_channels
+
         # Weights are initialized following pytorch approach
         self.conv = nn.Conv2d(
-            in_channels,
+            self.in_channels,
             out_channels,
             self.kernel_size,
             stride=self.stride,
@@ -566,6 +610,12 @@ class Conv2d(nn.Module):
             bias=bias,
         )
 
+        if conv_init == "kaiming":
+            nn.init.kaiming_normal_(self.conv.weight)
+
+        if weight_norm:
+            self.conv = nn.utils.weight_norm(self.conv)
+
     def forward(self, x):
         """Returns the output of the convolution.
 
@@ -575,7 +625,9 @@ class Conv2d(nn.Module):
             input to convolve. 2d or 4d tensors are expected.
 
         """
-        x = x.transpose(1, -1)
+        if not self.skip_transpose:
+            x = x.transpose(1, -1)
+
         if self.unsqueeze:
             x = x.unsqueeze(1)
 
@@ -584,19 +636,27 @@ class Conv2d(nn.Module):
                 x, self.kernel_size, self.dilation, self.stride
             )
 
+        elif self.padding == "causal":
+            num_pad = (self.kernel_size[0] - 1) * self.dilation[1]
+            x = F.pad(x, (0, 0, num_pad, 0))
+
         elif self.padding == "valid":
             pass
 
         else:
             raise ValueError(
-                "Padding must be 'same' or 'valid'. Got " + self.padding
+                "Padding must be 'same','valid' or 'causal'. Got "
+                + self.padding
             )
 
         wx = self.conv(x)
 
         if self.unsqueeze:
             wx = wx.squeeze(1)
-        wx = wx.transpose(1, -1)
+
+        if not self.skip_transpose:
+            wx = wx.transpose(1, -1)
+
         return wx
 
     def _manage_padding(
@@ -617,7 +677,7 @@ class Conv2d(nn.Module):
         stride: int
         """
         # Detecting input shape
-        L_in = x.shape[-1]
+        L_in = self.in_channels
 
         # Time padding
         padding_time = get_padding_elem(
@@ -635,8 +695,7 @@ class Conv2d(nn.Module):
         return x
 
     def _check_input(self, shape):
-        """Checks the input shape and returns the number of input channels.
-        """
+        """Checks the input shape and returns the number of input channels."""
 
         if len(shape) == 3:
             self.unsqueeze = True
@@ -649,7 +708,9 @@ class Conv2d(nn.Module):
             raise ValueError("Expected 3d or 4d inputs. Got " + len(shape))
 
         # Kernel size must be odd
-        if self.kernel_size[0] % 2 == 0 or self.kernel_size[1] % 2 == 0:
+        if not self.padding == "valid" and (
+            self.kernel_size[0] % 2 == 0 or self.kernel_size[1] % 2 == 0
+        ):
             raise ValueError(
                 "The field kernel size must be an odd number. Got %s."
                 % (self.kernel_size)
@@ -657,6 +718,10 @@ class Conv2d(nn.Module):
 
         return in_channels
 
+    def remove_weight_norm(self):
+        """Removes weight normalization at inference if used during training."""
+        self.conv = nn.utils.remove_weight_norm(self.conv)
+
 
 class Conv2dWithConstraint(Conv2d):
     """This function implements 2d convolution with kernel max-norm constaint.
@@ -768,6 +833,9 @@ class ConvTranspose1d(nn.Module):
     skip_transpose : bool
         If False, uses batch x time x channel convention of speechbrain.
         If True, uses batch x channel x time convention.
+    weight_norm : bool
+        If True, use weight normalization,
+        to be removed with self.remove_weight_norm() at inference
 
     Example
     -------
@@ -830,6 +898,7 @@ class ConvTranspose1d(nn.Module):
         groups=1,
         bias=True,
         skip_transpose=False,
+        weight_norm=False,
     ):
         super().__init__()
         self.kernel_size = kernel_size
@@ -883,6 +952,9 @@ class ConvTranspose1d(nn.Module):
             bias=bias,
         )
 
+        if weight_norm:
+            self.conv = nn.utils.weight_norm(self.conv)
+
     def forward(self, x, output_size=None):
         """Returns the output of the convolution.
 
@@ -909,8 +981,7 @@ class ConvTranspose1d(nn.Module):
         return wx
 
     def _check_input_shape(self, shape):
-        """Checks the input shape and returns the number of input channels.
-        """
+        """Checks the input shape and returns the number of input channels."""
 
         if len(shape) == 2:
             self.unsqueeze = True
@@ -926,6 +997,10 @@ class ConvTranspose1d(nn.Module):
 
         return in_channels
 
+    def remove_weight_norm(self):
+        """Removes weight normalization at inference if used during training."""
+        self.conv = nn.utils.remove_weight_norm(self.conv)
+
 
 class DepthwiseSeparableConv1d(nn.Module):
     """This class implements the depthwise separable 1d convolution.
@@ -1102,6 +1177,288 @@ class DepthwiseSeparableConv2d(nn.Module):
         return out
 
 
+class GaborConv1d(nn.Module):
+    """
+    This class implements 1D Gabor Convolutions from
+
+    Neil Zeghidour, Olivier Teboul, F{\'e}lix de Chaumont Quitry & Marco Tagliasacchi, "LEAF: A LEARNABLE FRONTEND
+    FOR AUDIO CLASSIFICATION", in Proc. of ICLR 2021 (https://arxiv.org/abs/2101.08596)
+
+    Arguments
+    ---------
+    out_channels : int
+        It is the number of output channels.
+    kernel_size: int
+        Kernel size of the convolutional filters.
+    stride : int
+        Stride factor of the convolutional filters. When the stride factor > 1,
+        a decimation in time is performed.
+    padding : str
+        (same, valid). If "valid", no padding is performed.
+        If "same" and stride is 1, output shape is the same as the input shape.
+    padding_mode : str
+        This flag specifies the type of padding. See torch.nn documentation
+        for more information.
+    sample_rate : int,
+        Sampling rate of the input signals. It is only used for sinc_conv.
+    min_freq : float
+        Lowest possible frequency (in Hz) for a filter
+    max_freq : float
+        Highest possible frequency (in Hz) for a filter
+    n_fft: int
+        number of FFT bins for initialization
+    normalize_energy: bool
+        whether to normalize energy at initialization. Default is False
+    bias : bool
+        If True, the additive bias b is adopted.
+    sort_filters: bool
+        whether to sort filters by center frequencies. Default is False
+    use_legacy_complex: bool
+        If False, torch.complex64 data type is used for gabor impulse responses
+        If True, computation is performed on two real-valued tensors
+    skip_transpose: bool
+        If False, uses batch x time x channel convention of speechbrain.
+        If True, uses batch x channel x time convention.
+
+    Example
+    -------
+    >>> inp_tensor = torch.rand([10, 8000])
+    >>> # 401 corresponds to a window of 25 ms at 16000 kHz
+    >>> gabor_conv = GaborConv1d(
+    ...     40, kernel_size=401, stride=1, in_channels=1
+    ... )
+    >>> #
+    >>> out_tensor = gabor_conv(inp_tensor)
+    >>> out_tensor.shape
+    torch.Size([10, 8000, 40])
+    """
+
+    def __init__(
+        self,
+        out_channels,
+        kernel_size,
+        stride,
+        input_shape=None,
+        in_channels=None,
+        padding="same",
+        padding_mode="constant",
+        sample_rate=16000,
+        min_freq=60.0,
+        max_freq=None,
+        n_fft=512,
+        normalize_energy=False,
+        bias=False,
+        sort_filters=False,
+        use_legacy_complex=False,
+        skip_transpose=False,
+    ):
+        super(GaborConv1d, self).__init__()
+        self.filters = out_channels // 2
+        self.kernel_size = kernel_size
+        self.stride = stride
+        self.padding = padding
+        self.padding_mode = padding_mode
+        self.sort_filters = sort_filters
+        self.sample_rate = sample_rate
+        self.min_freq = min_freq
+        if max_freq is None:
+            max_freq = sample_rate / 2
+        self.max_freq = max_freq
+        self.n_fft = n_fft
+        self.normalize_energy = normalize_energy
+        self.use_legacy_complex = use_legacy_complex
+        self.skip_transpose = skip_transpose
+
+        if input_shape is None and in_channels is None:
+            raise ValueError("Must provide one of input_shape or in_channels")
+
+        if in_channels is None:
+            in_channels = self._check_input_shape(input_shape)
+
+        self.kernel = nn.Parameter(self._initialize_kernel())
+        if bias:
+            self.bias = torch.nn.Parameter(torch.ones(self.filters * 2,))
+        else:
+            self.bias = None
+
+    def forward(self, x):
+        """Returns the output of the Gabor convolution.
+
+        Arguments
+        ---------
+        x : torch.Tensor (batch, time, channel)
+            input to convolve.
+        """
+        if not self.skip_transpose:
+            x = x.transpose(1, -1)
+
+        unsqueeze = x.ndim == 2
+        if unsqueeze:
+            x = x.unsqueeze(1)
+
+        kernel = self._gabor_constraint(self.kernel)
+        if self.sort_filters:
+            idxs = torch.argsort(kernel[:, 0])
+            kernel = kernel[idxs, :]
+
+        filters = self._gabor_filters(kernel)
+        if not self.use_legacy_complex:
+            temp = torch.view_as_real(filters)
+            real_filters = temp[:, :, 0]
+            img_filters = temp[:, :, 1]
+        else:
+            real_filters = filters[:, :, 0]
+            img_filters = filters[:, :, 1]
+        stacked_filters = torch.cat(
+            [real_filters.unsqueeze(1), img_filters.unsqueeze(1)], dim=1
+        )
+        stacked_filters = torch.reshape(
+            stacked_filters, (2 * self.filters, self.kernel_size)
+        )
+        stacked_filters = stacked_filters.unsqueeze(1)
+
+        if self.padding == "same":
+            x = self._manage_padding(x, self.kernel_size)
+        elif self.padding == "valid":
+            pass
+        else:
+            raise ValueError(
+                "Padding must be 'same' or 'valid'. Got " + self.padding
+            )
+
+        output = F.conv1d(
+            x, stacked_filters, bias=self.bias, stride=self.stride, padding=0
+        )
+        if not self.skip_transpose:
+            output = output.transpose(1, -1)
+        return output
+
+    def _gabor_constraint(self, kernel_data):
+        mu_lower = 0.0
+        mu_upper = math.pi
+        sigma_lower = (
+            4
+            * torch.sqrt(
+                2.0 * torch.log(torch.tensor(2.0, device=kernel_data.device))
+            )
+            / math.pi
+        )
+        sigma_upper = (
+            self.kernel_size
+            * torch.sqrt(
+                2.0 * torch.log(torch.tensor(2.0, device=kernel_data.device))
+            )
+            / math.pi
+        )
+        clipped_mu = torch.clamp(
+            kernel_data[:, 0], mu_lower, mu_upper
+        ).unsqueeze(1)
+        clipped_sigma = torch.clamp(
+            kernel_data[:, 1], sigma_lower, sigma_upper
+        ).unsqueeze(1)
+        return torch.cat([clipped_mu, clipped_sigma], dim=-1)
+
+    def _gabor_filters(self, kernel):
+        t = torch.arange(
+            -(self.kernel_size // 2),
+            (self.kernel_size + 1) // 2,
+            dtype=kernel.dtype,
+            device=kernel.device,
+        )
+        if not self.use_legacy_complex:
+            return gabor_impulse_response(
+                t, center=kernel[:, 0], fwhm=kernel[:, 1]
+            )
+        else:
+            return gabor_impulse_response_legacy_complex(
+                t, center=kernel[:, 0], fwhm=kernel[:, 1]
+            )
+
+    def _manage_padding(self, x, kernel_size):
+        # this is the logic that gives correct shape that complies
+        # with the original implementation at https://github.com/google-research/leaf-audio
+
+        def get_padding_value(kernel_size):
+            """Gets the number of elements to pad."""
+            kernel_sizes = (kernel_size,)
+            from functools import reduce
+            from operator import __add__
+
+            conv_padding = reduce(
+                __add__,
+                [
+                    (k // 2 + (k - 2 * (k // 2)) - 1, k // 2)
+                    for k in kernel_sizes[::-1]
+                ],
+            )
+            return conv_padding
+
+        pad_value = get_padding_value(kernel_size)
+        x = F.pad(x, pad_value, mode=self.padding_mode, value=0)
+        return x
+
+    def _mel_filters(self):
+        def _mel_filters_areas(filters):
+            peaks, _ = torch.max(filters, dim=1, keepdim=True)
+            return (
+                peaks
+                * (torch.sum((filters > 0).float(), dim=1, keepdim=True) + 2)
+                * np.pi
+                / self.n_fft
+            )
+
+        mel_filters = torchaudio.functional.melscale_fbanks(
+            n_freqs=self.n_fft // 2 + 1,
+            f_min=self.min_freq,
+            f_max=self.max_freq,
+            n_mels=self.filters,
+            sample_rate=self.sample_rate,
+        )
+        mel_filters = mel_filters.transpose(1, 0)
+        if self.normalize_energy:
+            mel_filters = mel_filters / _mel_filters_areas(mel_filters)
+        return mel_filters
+
+    def _gabor_params_from_mels(self):
+        coeff = torch.sqrt(2.0 * torch.log(torch.tensor(2.0))) * self.n_fft
+        sqrt_filters = torch.sqrt(self._mel_filters())
+        center_frequencies = torch.argmax(sqrt_filters, dim=1)
+        peaks, _ = torch.max(sqrt_filters, dim=1, keepdim=True)
+        half_magnitudes = peaks / 2.0
+        fwhms = torch.sum((sqrt_filters >= half_magnitudes).float(), dim=1)
+        output = torch.cat(
+            [
+                (center_frequencies * 2 * np.pi / self.n_fft).unsqueeze(1),
+                (coeff / (np.pi * fwhms)).unsqueeze(1),
+            ],
+            dim=-1,
+        )
+        return output
+
+    def _initialize_kernel(self):
+        return self._gabor_params_from_mels()
+
+    def _check_input_shape(self, shape):
+        """Checks the input shape and returns the number of input channels."""
+
+        if len(shape) == 2:
+            in_channels = 1
+        elif len(shape) == 3:
+            in_channels = 1
+        else:
+            raise ValueError(
+                "GaborConv1d expects 2d or 3d inputs. Got " + str(len(shape))
+            )
+
+        # Kernel size must be odd
+        if self.kernel_size % 2 == 0:
+            raise ValueError(
+                "The field kernel size must be an odd number. Got %s."
+                % (self.kernel_size)
+            )
+        return in_channels
+
+
 def get_padding_elem(L_in: int, stride: int, kernel_size: int, dilation: int):
     """This function computes the number of elements to add for zero-padding.
 
