@@ -37,7 +37,7 @@ class CLSTM(torch.nn.Module):
     return_hidden : bool, optional
         It True, the function returns the last hidden layer.
     bidirectional : bool, optional
-        If True, a bidirectioal model that scans the sequence both
+        If True, a bidirectional model that scans the sequence both
         right-to-left and left-to-right is used (default False).
     init_criterion : str , optional
         (glorot, he).
@@ -195,7 +195,7 @@ class CLSTM_Layer(torch.nn.Module):
     dropout : float, optional
         It is the dropout factor (must be between 0 and 1) (default 0.0).
     bidirectional : bool, optional
-        If True, a bidirectioal model that scans the sequence both
+        If True, a bidirectional model that scans the sequence both
         right-to-left and left-to-right is used (default False).
     init_criterion : str, optional
         (glorot, he).
@@ -253,12 +253,12 @@ class CLSTM_Layer(torch.nn.Module):
             self.batch_size = self.batch_size * 2
 
         # Initial state
-        self.h_init = torch.zeros(1, self.hidden_size * 2, requires_grad=False)
+        self.register_buffer("h_init", torch.zeros(1, self.hidden_size * 2))
 
         # Preloading dropout masks (gives some speed improvement)
         self._init_drop(self.batch_size)
 
-        # Initilizing dropout
+        # Initializing dropout
         self.drop = torch.nn.Dropout(p=self.dropout, inplace=False)
 
         self.drop_mask_te = torch.tensor([1.0]).float()
@@ -310,7 +310,7 @@ class CLSTM_Layer(torch.nn.Module):
         ct = self.h_init
 
         # Sampling dropout mask
-        drop_mask = self._sample_drop_mask()
+        drop_mask = self._sample_drop_mask(w)
 
         # Loop over time axis
         for k in range(w.shape[1]):
@@ -343,11 +343,12 @@ class CLSTM_Layer(torch.nn.Module):
         self.N_drop_masks = 16000
         self.drop_mask_cnt = 0
 
-        self.drop_masks = self.drop(
-            torch.ones(self.N_drop_masks, self.hidden_size * 2)
-        ).data
+        self.register_buffer(
+            "drop_masks",
+            self.drop(torch.ones(self.N_drop_masks, self.hidden_size * 2)).data,
+        )
 
-    def _sample_drop_mask(self,):
+    def _sample_drop_mask(self, w):
         """Selects one of the pre-defined dropout masks
         """
 
@@ -357,7 +358,9 @@ class CLSTM_Layer(torch.nn.Module):
             if self.drop_mask_cnt + self.batch_size > self.N_drop_masks:
                 self.drop_mask_cnt = 0
                 self.drop_masks = self.drop(
-                    torch.ones(self.N_drop_masks, self.hidden_size * 2,)
+                    torch.ones(
+                        self.N_drop_masks, self.hidden_size * 2, device=w.device
+                    )
                 ).data
 
             # Sampling the mask
@@ -367,6 +370,7 @@ class CLSTM_Layer(torch.nn.Module):
             self.drop_mask_cnt = self.drop_mask_cnt + self.batch_size
 
         else:
+            self.drop_mask_te = self.drop_mask_te.to(w.device)
             drop_mask = self.drop_mask_te
 
         return drop_mask
@@ -383,7 +387,7 @@ class CLSTM_Layer(torch.nn.Module):
 
             if self.training:
                 self.drop_masks = self.drop(
-                    torch.ones(self.N_drop_masks, self.hidden_size * 2,)
+                    torch.ones(self.N_drop_masks, self.hidden_size * 2)
                 ).data
 
 
@@ -411,7 +415,7 @@ class CRNN(torch.nn.Module):
     return_hidden : bool, optional
         It True, the function returns the last hidden layer (default False).
     bidirectional : bool, optional
-        If True, a bidirectioal model that scans the sequence both
+        If True, a bidirectional model that scans the sequence both
         right-to-left and left-to-right is used (default False).
     init_criterion : str , optional
         (glorot, he).
@@ -573,7 +577,7 @@ class CRNN_Layer(torch.nn.Module):
     dropout : float, optional
         It is the dropout factor (must be between 0 and 1) (default 0.0).
     bidirectional : bool, optional
-        If True, a bidirectioal model that scans the sequence both
+        If True, a bidirectional model that scans the sequence both
         right-to-left and left-to-right is used (default False).
     init_criterion : str , optional
         (glorot, he).
@@ -631,12 +635,12 @@ class CRNN_Layer(torch.nn.Module):
             self.batch_size = self.batch_size * 2
 
         # Initial state
-        self.h_init = torch.zeros(1, self.hidden_size * 2, requires_grad=False)
+        self.register_buffer("h_init", torch.zeros(1, self.hidden_size * 2))
 
         # Preloading dropout masks (gives some speed improvement)
         self._init_drop(self.batch_size)
 
-        # Initilizing dropout
+        # Initializing dropout
         self.drop = torch.nn.Dropout(p=self.dropout, inplace=False)
 
         self.drop_mask_te = torch.tensor([1.0]).float()
@@ -692,7 +696,7 @@ class CRNN_Layer(torch.nn.Module):
         hiddens = []
 
         # Sampling dropout mask
-        drop_mask = self._sample_drop_mask()
+        drop_mask = self._sample_drop_mask(w)
 
         # Loop over time axis
         for k in range(w.shape[1]):
@@ -715,12 +719,13 @@ class CRNN_Layer(torch.nn.Module):
         self.N_drop_masks = 16000
         self.drop_mask_cnt = 0
 
-        self.drop_masks = self.drop(
-            torch.ones(self.N_drop_masks, self.hidden_size * 2,)
-        ).data
+        self.register_buffer(
+            "drop_masks",
+            self.drop(torch.ones(self.N_drop_masks, self.hidden_size * 2)).data,
+        )
 
-    def _sample_drop_mask(self,):
-        """Selects one of the pre-defined dropout masks.
+    def _sample_drop_mask(self, w):
+        """Selects one of the pre-defined dropout masks
         """
 
         if self.training:
@@ -729,7 +734,9 @@ class CRNN_Layer(torch.nn.Module):
             if self.drop_mask_cnt + self.batch_size > self.N_drop_masks:
                 self.drop_mask_cnt = 0
                 self.drop_masks = self.drop(
-                    torch.ones(self.N_drop_masks, self.hidden_size * 2,)
+                    torch.ones(
+                        self.N_drop_masks, self.hidden_size * 2, device=w.device
+                    )
                 ).data
 
             # Sampling the mask
@@ -739,6 +746,7 @@ class CRNN_Layer(torch.nn.Module):
             self.drop_mask_cnt = self.drop_mask_cnt + self.batch_size
 
         else:
+            self.drop_mask_te = self.drop_mask_te.to(w.device)
             drop_mask = self.drop_mask_te
 
         return drop_mask
@@ -755,7 +763,7 @@ class CRNN_Layer(torch.nn.Module):
 
             if self.training:
                 self.drop_masks = self.drop(
-                    torch.ones(self.N_drop_masks, self.hidden_size * 2,)
+                    torch.ones(self.N_drop_masks, self.hidden_size * 2)
                 ).data
 
 
@@ -798,7 +806,7 @@ class CLiGRU(torch.nn.Module):
     return_hidden : bool
         If True, the function returns the last hidden layer.
     bidirectional : bool
-        If True, a bidirectioal model that scans the sequence both
+        If True, a bidirectional model that scans the sequence both
         right-to-left and left-to-right is used.
     init_criterion : str , optional
         (glorot, he).
@@ -966,7 +974,7 @@ class CLiGRU_Layer(torch.nn.Module):
     dropout : float
         It is the dropout factor (must be between 0 and 1).
     bidirectional : bool
-        If True, a bidirectioal model that scans the sequence both
+        If True, a bidirectional model that scans the sequence both
         right-to-left and left-to-right is used.
     init_criterion : str , optional
         (glorot, he).
@@ -1031,7 +1039,7 @@ class CLiGRU_Layer(torch.nn.Module):
 
         if self.normalization == "batchnorm":
             self.norm = CBatchNorm(
-                input_size=hidden_size * 2, dim=-1, momentum=0.05,
+                input_size=hidden_size * 2, dim=-1, momentum=0.05
             )
             self.normalize = True
 
@@ -1045,12 +1053,12 @@ class CLiGRU_Layer(torch.nn.Module):
             self.normalize = True
 
         # Initial state
-        self.h_init = torch.zeros(1, self.hidden_size * 2, requires_grad=False)
+        self.register_buffer("h_init", torch.zeros(1, self.hidden_size * 2))
 
         # Preloading dropout masks (gives some speed improvement)
         self._init_drop(self.batch_size)
 
-        # Initilizing dropout
+        # Initializing dropout
         self.drop = torch.nn.Dropout(p=self.dropout, inplace=False)
 
         self.drop_mask_te = torch.tensor([1.0]).float()
@@ -1111,7 +1119,7 @@ class CLiGRU_Layer(torch.nn.Module):
         hiddens = []
 
         # Sampling dropout mask
-        drop_mask = self._sample_drop_mask()
+        drop_mask = self._sample_drop_mask(w)
 
         # Loop over time axis
         for k in range(w.shape[1]):
@@ -1139,20 +1147,24 @@ class CLiGRU_Layer(torch.nn.Module):
         self.N_drop_masks = 16000
         self.drop_mask_cnt = 0
 
-        self.drop_masks = self.drop(
-            torch.ones(self.N_drop_masks, self.hidden_size * 2)
-        ).data
+        self.register_buffer(
+            "drop_masks",
+            self.drop(torch.ones(self.N_drop_masks, self.hidden_size * 2)).data,
+        )
 
-    def _sample_drop_mask(self,):
-        """Selects one of the pre-defined dropout masks.
+    def _sample_drop_mask(self, w):
+        """Selects one of the pre-defined dropout masks
         """
+
         if self.training:
 
             # Sample new masks when needed
             if self.drop_mask_cnt + self.batch_size > self.N_drop_masks:
                 self.drop_mask_cnt = 0
                 self.drop_masks = self.drop(
-                    torch.ones(self.N_drop_masks, self.hidden_size * 2,)
+                    torch.ones(
+                        self.N_drop_masks, self.hidden_size * 2, device=w.device
+                    )
                 ).data
 
             # Sampling the mask
@@ -1162,6 +1174,7 @@ class CLiGRU_Layer(torch.nn.Module):
             self.drop_mask_cnt = self.drop_mask_cnt + self.batch_size
 
         else:
+            self.drop_mask_te = self.drop_mask_te.to(w.device)
             drop_mask = self.drop_mask_te
 
         return drop_mask
