@@ -58,6 +58,8 @@ import inspect
 import shutil
 import logging
 import warnings
+from packaging import version
+import speechbrain.utils._workarounds as __wa
 
 logger = logging.getLogger(__name__)
 
@@ -157,18 +159,37 @@ def torch_parameter_transfer(obj, path, device):
 
 
 # These dicts are indexed by class and hold the default checkpoints methods
-DEFAULT_LOAD_HOOKS = {
-    torch.nn.Module: torch_recovery,
-    torch.optim.Optimizer: torch_recovery,
-    torch.optim.lr_scheduler._LRScheduler: torch_recovery,
-    torch.optim.lr_scheduler.ReduceLROnPlateau: torch_recovery,
-}
-DEFAULT_SAVE_HOOKS = {
-    torch.nn.Module: torch_save,
-    torch.optim.Optimizer: torch_save,
-    torch.optim.lr_scheduler._LRScheduler: torch_save,
-    torch.optim.lr_scheduler.ReduceLROnPlateau: torch_save,
-}
+if version.parse(torch.__version__) < version.parse("2.0.0"):
+    DEFAULT_LOAD_HOOKS = {
+        torch.nn.Module: torch_recovery,
+        torch.optim.Optimizer: torch_recovery,
+        torch.optim.lr_scheduler._LRScheduler: torch_recovery,
+        torch.optim.lr_scheduler.ReduceLROnPlateau: torch_recovery,
+        torch.cuda.amp.grad_scaler.GradScaler: torch_recovery,
+    }
+    DEFAULT_SAVE_HOOKS = {
+        torch.nn.Module: torch_save,
+        torch.optim.Optimizer: torch_save,
+        torch.optim.lr_scheduler._LRScheduler: torch_save,
+        torch.optim.lr_scheduler.ReduceLROnPlateau: torch_save,
+        torch.cuda.amp.grad_scaler.GradScaler: torch_save,
+    }
+else:
+    DEFAULT_LOAD_HOOKS = {
+        torch.nn.Module: torch_recovery,
+        torch.optim.Optimizer: torch_recovery,
+        torch.optim.lr_scheduler.LRScheduler: torch_recovery,
+        torch.optim.lr_scheduler.ReduceLROnPlateau: torch_recovery,
+        torch.cuda.amp.grad_scaler.GradScaler: torch_recovery,
+    }
+    DEFAULT_SAVE_HOOKS = {
+        torch.nn.Module: torch_save,
+        torch.optim.Optimizer: torch_save,
+        torch.optim.lr_scheduler.LRScheduler: torch_save,
+        torch.optim.lr_scheduler.ReduceLROnPlateau: torch_save,
+        torch.cuda.amp.grad_scaler.GradScaler: torch_save,
+    }
+
 DEFAULT_TRANSFER_HOOKS = {
     torch.nn.Module: torch_parameter_transfer,
 }
@@ -186,6 +207,10 @@ except ImportError:
     # SentencePiece not loaded, fine!
     pass
 
+# Add workarounds:
+DEFAULT_SAVE_HOOKS[torch.optim.lr_scheduler.CyclicLR] = __wa._cycliclrsaver
+DEFAULT_LOAD_HOOKS[torch.optim.lr_scheduler.CyclicLR] = __wa._cycliclrloader
+
 
 def mark_as_saver(method):
     """Method decorator which marks given method as the checkpoint saving hook.
