@@ -299,6 +299,8 @@ def download_file(
         If True, replaces the existing files.
     """
     try:
+        # make sure all processing reached here before main preocess create dest_dir
+        sb.utils.distributed.ddp_barrier()
         if sb.utils.distributed.if_main_process():
 
             class DownloadProgressBar(tqdm.tqdm):
@@ -416,7 +418,7 @@ def batch_pad_right(tensors: list, mode="constant", value=0):
         return tensors[0].unsqueeze(0), torch.tensor([1.0])
 
     if not (
-        any(
+        all(
             [tensors[i].ndim == tensors[0].ndim for i in range(1, len(tensors))]
         )
     ):
