@@ -1,3 +1,8 @@
+"""Library for checking the torchaudio backend.
+
+Authors
+ * Mirco Ravanelli 2021
+"""
 import platform
 import logging
 import torchaudio
